@@ -49,6 +49,13 @@ plm <-  function(formula, data, subset, na.action,
   if (!any(is.na(model))) model <- match.arg(model)
   random.method <- match.arg(random.method)
   inst.method <- match.arg(inst.method)
+  
+  # input checks for FD model
+  # give informative error messages as described in footnote in vignette
+  if (!is.na(model) && model == "fd") {
+    if (effect == "time") stop("effect = \"time\" for first-difference model meaningless because cross-sections do not generally have a natural ordering")
+    if (effect == "twoways") stop("effect = \"twoways\" is not defined for first-difference models")
+  }
 
   # deprecated, pht is no longer maintained
   if (!is.na(model) && model == "ht"){
@@ -71,7 +78,7 @@ plm <-  function(formula, data, subset, na.action,
   # Check whether data is a pdata.frame and if not create it
   orig_rownames <- row.names(data)
   if (!inherits(data, "pdata.frame")) data <- pdata.frame(data, index)
-  # Create a Formula object if necessary
+  # Create a pFormula object if necessary
   if (!inherits(formula, "pFormula")) formula <- pFormula(formula)
 
   # in case of 2part formula, check whether the second part should be
@@ -114,8 +121,9 @@ plm <-  function(formula, data, subset, na.action,
 plm.fit <- function(formula, data, model, effect, random.method, random.dfcor, inst.method){
   # if a random effect model is estimated, compute the error components
   if (model == "random"){
-    pdim <- pdim(data)
-    is.balanced <- pdim$balanced
+    # pdim <- pdim(data)
+    # is.balanced <- pdim$balanced
+    is.balanced <- is.pbalanced(data)
     estec <- ercomp(formula, data, effect, method = random.method, dfcor = random.dfcor)
     sigma2 <- estec$sigma2
     theta <- estec$theta
@@ -157,10 +165,12 @@ plm.fit <- function(formula, data, model, effect, random.method, random.dfcor, i
   else W <- NULL
   # compute the estimation
   result <- mylm(y, X, W)
-  # in case of a within estimation, correct the degrees of freedom
+  
   df <- df.residual(result)
   vcov <- result$vcov
-
+  aliased <- result$aliased
+  
+  # in case of a within estimation, correct the degrees of freedom
   if (model == "within"){
     pdim <- pdim(data)
     card.fixef <- switch(effect,
@@ -171,6 +181,7 @@ plm.fit <- function(formula, data, model, effect, random.method, random.dfcor, i
     df <- df.residual(result) - card.fixef
     vcov <- result$vcov * df.residual(result) / df
   }
+  
   result <- list(coefficients = coef(result),
                  vcov         = vcov,
                  residuals    = resid(result),
@@ -178,10 +189,13 @@ plm.fit <- function(formula, data, model, effect, random.method, random.dfcor, i
                  formula      = formula,
                  model        = data)
   if (model == "random") result$ercomp <- estec
+  result$assign <- attr(X, "assign")
+  result$contrasts <- attr(X, "contrasts")
   result$args <- list(model = model, effect = effect)
+  result$aliased <- aliased
   class(result) <- c("plm", "panelmodel")
   result
-}    
+}
 
 mylm <- function(y, X, W = NULL){
   names.X <- colnames(X)
@@ -189,8 +203,10 @@ mylm <- function(y, X, W = NULL){
       result <- lm(y ~ X - 1)
   else
       result <- twosls(y, X, W)
-  if (any(is.na(coef(result)))){
-    na.coef <- is.na(coef(result))
+  
+  na.coef <- is.na(coef(result))
+  if (any(na.coef)){
+    # warning("Coefficient(s) '", paste((names.X)[na.coef], collapse = ", "), "' could not be estimated and is (are) dropped.")
     X <- X[, !na.coef, drop = FALSE]
     if (is.null(W)) result <- lm(y ~ X - 1)
     else result <- twosls(y, X, W)
@@ -199,6 +215,10 @@ mylm <- function(y, X, W = NULL){
   result$X <- X
   result$y <- y
   result$W <- W
+  # aliased is an element of summary.lm-objects:
+  # since plm drops aliased coefs, store this info in plm object
+  result$aliased <- na.coef
+  names(result$aliased) <- names.X
   names(result$coefficients) <- colnames(result$vcov) <-
     rownames(result$vcov) <- colnames(X)
   result
